/*
   These test functions fill an array of size n, where n is a command line parameter,
   with a random permutation of the integers from 1 to n, and then the random array
   is sorted. The time it takes to sort the array is (roughly) calculated.
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>      // needed for time_t         and time()
#include <sys/time.h>  // needed for struct timeval and gettimeofday()
#include <windows.h>   // needed for LARGE_INTEGER  and QueryPerformanceCounter()
#include "Timing.h"    // uses Win32 GetSystemTime() and GetThreadTimes()
#include "MergeSort.h"
#include "InsertionSort.h"

/*
   This function will randomly permute the elements of the array.
*/
void permutation(int* a, int n)
{
   int i, upperBound, temp;

   srand( time(NULL) );

   upperBound = n;
   // The array from indices 0 to upperBound-1 is the "jar" to draw randomly
   // from, and the array from indices upperBound to n-1 is the list of
   // numbers that have already been drawn out of the "jar".
   for (i = 0; i < n; i++)
   {
      // pick a slot in the "jar" part of the array
      int randomIndex = rand() % upperBound;

      // swap a[upperBound-1] with a[randomIndex]
      // (that is, move an item from the "jar" part of the array
      // to the list part of the array)
      temp = a[upperBound-1];
      a[upperBound-1] = a[randomIndex];
      a[randomIndex] = temp;

      // lower the upperbound
      upperBound--;  // lower the boundary of the "jar" part of the array
   }
}//permutation()


/*
   This function prints an array in rows of 10.
*/
void printTheArray(int* a, int n)
{
   int i;
   printf("\nHere is the array.\n");
   for (i = 0; i < n; i++)
   {
      printf(" %d", a[i]);
      if ((i+1)%10 == 0) printf("\n");
   }
   printf("\n");
}//printTheArray()


int main(int argc, char** argv)
{
   // data structures for three different timing systems
   LARGE_INTEGER clockFrequency;                  // clock ticks per second
   LARGE_INTEGER clockStart, clockEnd;            // time in clock ticks
   double clockElapsed;
   TimingObject timeStart, timeEnd, timeElapsed;  // time in milli-seconds
   struct timeval startTime, endTime;             // time in micro-second
   int elapsedTime;

   int *integerArray, *copiedArray;
   int i;

   int n;
   if (argc > 1)
   {
      n = atoi( argv[1] );  // get parameter from command line
   }
   else
   {
      printf("Usage: sort <n>");
      return 0;
   }

   integerArray = malloc(n*sizeof(int));  // array that will hold the permutation.
   if (integerArray == NULL)  // error check
   {
      fprintf(stderr, "Sort: not enough memory for array of size %d.\n", n);
      return -1;
   }
   copiedArray = malloc(n*sizeof(int));  // array that will hold a copy of the permutation.
   if (copiedArray == NULL)  // error check
   {
      fprintf(stderr, "Sort: not enough memory for array of size %d.\n", n);
      return -1;
   }

   // initialize the array with the numbers for 1 to n
   for (i = 0; i < n; i++) integerArray[i] = i+1;

   // the permutation() function will randomly permute
   // the contents of the array
   permutation(integerArray, n);

   // make a copy of the random permutation
   for (i = 0; i < n; i++) copiedArray[i] = integerArray[i];

   // if n is small enough, print out the array.
   if (n <= 256)
   {
      printTheArray(integerArray, n);
   }

   // get number of clock ticks per second
   QueryPerformanceFrequency(&clockFrequency);

   // sort the array using Mergesort
   //(and measure time three different ways)
   gettimeofday(&startTime, NULL);       // micro-second timer
   getThreadTime(&timeStart);            // milli-second timer
   QueryPerformanceCounter(&clockStart); // clock-tick timer
   mergeSort( integerArray, 0, n-1 );
   QueryPerformanceCounter(&clockEnd);
   getThreadTime(&timeEnd);
   gettimeofday(&endTime, NULL);

   printf("Mergesort took\n");

   // Compute the sorting time using the high resolution timer.
   clockElapsed = clockEnd.QuadPart - clockStart.QuadPart;
   printf("\t%f seconds,\n", clockElapsed/clockFrequency.QuadPart);

   // Compute the sorting time using the low resolution timer
   // that includes "user time" measurements.
   computeElapsedTime( &timeStart, &timeEnd, &timeElapsed );
   printTimingObject( &timeElapsed );

   // Compute the sorting time using the timer that is compatible with Linux.
   elapsedTime = (  endTime.tv_sec*1000000 +  endTime.tv_usec)
               - (startTime.tv_sec*1000000 + startTime.tv_usec);
   printf("\t%d micro-seconds,\n", elapsedTime);

   printf("to sort an array of size %d.\n", n);


   // sort the copy of the array using Insertion sort
   // (and measure time three different ways)
   gettimeofday(&startTime, NULL);       // micro-second timer
   getThreadTime(&timeStart);            // milli-second timer
   QueryPerformanceCounter(&clockStart); // clock-tick timer
   insertionSort( copiedArray, 0, n-1 );
   QueryPerformanceCounter(&clockEnd);
   getThreadTime(&timeEnd);
   gettimeofday(&endTime, NULL);

   printf("Insertion sort took\n");

   // Compute the sorting time using the high resolution timer.
   clockElapsed = clockEnd.QuadPart - clockStart.QuadPart;
   printf("\t%f seconds,\n", clockElapsed/clockFrequency.QuadPart);

   // Compute the sorting time using the low resolution timer
   // that includes "user time" measurements.
   computeElapsedTime( &timeStart, &timeEnd, &timeElapsed );
   printTimingObject( &timeElapsed );

   // Compute the sorting time using the timer that is compatible with Linux.
   elapsedTime = (  endTime.tv_sec*1000000 +  endTime.tv_usec)
               - (startTime.tv_sec*1000000 + startTime.tv_usec);
   printf("\t%d micro-seconds,\n", elapsedTime);

   printf("to sort the same array of size %d.\n", n);

   // if n is small enough, print out the array.
   if (n <= 256)
   {
      printTheArray(integerArray, n);
   }

   return 0;
}//main()
